@@ -11,7 +11,7 @@ use warnings;
 
 our @ISA = qw(Exporter);
 our @EXPORT = qw(bg_eval bg_qx);
-our $VERSION = '0.52';
+our $VERSION = '0.53';
 
 sub _choose_protocol {
   if (CONFIG_module('YAML')) {
@@ -35,13 +35,19 @@ sub bg_eval (&;@) {
     croak "Forks::Super: bg_eval call requires either YAML or JSON\n";
   }
   my ($code, @other_options) = @_;
+  my %other_options;
   if (@other_options > 0 && ref $other_options[0] eq 'HASH') {
-    @other_options = %{$other_options[0]};
+      %other_options = %{$other_options[0]};
+  } else {
+      %other_options = @other_options;
+  }
+  if (defined($other_options{daemon}) && $other_options{daemon}) {
+      croak "Forks::Super::bg_eval: daemon option not allowed on bg_eval call";
   }
 
   if ($Forks::Super::SysInfo::SLEEP_ALARM_COMPATIBLE <= 0) {
     # timeout, expiration are incompatible with bg_eval
-    foreach (@other_options) {
+    foreach (keys %other_options) {
       if ($_ eq "timeout" || $_ eq "expiration") {
 	croak "Forks::Super::bg_eval: ",
 	  "$_ option not allowed because ",
@@ -57,7 +63,7 @@ sub bg_eval (&;@) {
   $result = Forks::Super::Tie::BackgroundScalar->new(
       'eval', $code, 
       protocol => $proto,
-      @other_options);
+      %other_options);
   if ($$ != $p) {
       # a WTF observed on Windows
       croak "Forks::Super::bg_eval: ",
@@ -68,13 +74,19 @@ sub bg_eval (&;@) {
 
 sub bg_qx {
   my ($command, @other_options) = @_;
+  my %other_options;
   if (@other_options > 0 && ref $other_options[0] eq 'HASH') {
-    @other_options = %{$other_options[0]};
+      %other_options = %{$other_options[0]};
+  } else {
+      %other_options = @other_options;
   }
 
+  if (defined($other_options{daemon}) && $other_options{daemon}) {
+      croak "Forks::Super::bg_qx: daemon option not allowed on bg_qx call";
+  }
   if ($Forks::Super::SysInfo::SLEEP_ALARM_COMPATIBLE <= 0) {
     # timeout, expiration are incompatible with bg_qx
-    foreach (@other_options) {
+    foreach (keys %other_options) {
       if ($_ eq "timeout" || $_ eq "expiration") {
 	croak "Forks::Super::bg_qx: ",
 	  "$_ option not allowed because ",
@@ -88,7 +100,7 @@ sub bg_qx {
 
   require Forks::Super::Tie::BackgroundScalar;
   $result =  Forks::Super::Tie::BackgroundScalar->new(
-      'qx', $command, @other_options);
+      'qx', $command, %other_options);
   if ($$ != $p) {
       # a WTF observed on Windows
       croak "Forks::Super::bg_qx: ",
