@@ -24,8 +24,6 @@ use IO::Pipe;
 use IO::Handle;
 our @ISA = qw(IO::Pipe IO::Handle);
 
-our $DEBUG = defined($ENV{XFH}) && $ENV{XFH} > 1;
-
 sub TIEHANDLE {
   my ($class, $real_pipe, $glob) = @_;
   $$glob->{DELEGATE} = $real_pipe;
@@ -178,6 +176,16 @@ sub DESTROY {
   return;
 }
 
+sub ___UNTIE {
+    my ($self, $existing_inner_references) = @_;
+    if ($existing_inner_references > 1) {
+	if (!$Forks::Super::Job::INSIDE_END_QUEUE) {
+	    warn "untie attempted while ", $existing_inner_references,
+	    	" still exist";
+	}
+    }
+}
+
 #
 # when you call a method on a glob that is tied to a 
 # Forks::Super::Tie::IPCSocketHandle , the method should be invoked
